#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import logging
import json
import sys
import urllib2

import yaml

format_list = {'json', 'yaml'}
# Make up some resonable data returned from each service
data = {
    'master': {
        'autopilot': {
            'version': '1.3.1+13.10.20131003.1-0ubuntu2~fginther.1',
            'files': [
                '/path/to/changes',
                '/path/to/dsc',
                '/path/to/diff.gz']}},
    'bsbuilder': {
        'result': 'PASSED',
        'version': '1.3.1+13.10.20131003.1-0ubuntu2~fginther.1',
        'packages': [
            'autopilot-desktop',
            'autopilot-touch',
            'python-autopilot',
            'python-autopilot-test',
            'python-autopilot-trace',
            'python-autopilot-vis'],
        'ppa': 'ppa:team/ppa'},
    'image_builder': {
        'result': 'PASSED',
        'image': '/path/to/image'},
    'test_runner': {
        'result': 'PASSED',
        'artifacts': [
            '/path/to/artifact1',
            '/path/to/artifact2']},
    'publisher': {
        'result': 'PASSED',
        'image': '/path/to/image'},
}

logger = logging.getLogger('lander_service_wrapper')
stdout_handler = logging.StreamHandler(stream=sys.stdout)
formatter = logging.Formatter('%(levelname)s: %(message)s')
stdout_handler.setFormatter(formatter)
logger.addHandler(stdout_handler)


def _post(url, data):
    req = urllib2.Request(url)
    req.add_header('Content-Type', 'application/json')
    resp = urllib2.urlopen(req, json.dumps(data))
    return resp.headers['location']


def _get(url):
    resp = urllib2.urlopen(url)
    resp = resp.read()
    return json.loads(resp)


def _handle_ppa_assigner(args):
    config = _get_config(args)
    url = '%s/api/v1/ppa/' % config['ppa_assigner_url']
    loc = _post(url, {'ticket_id': config['request_id']})
    data = _get(loc)
    result = {
        'result': 'PASSED',
        'ppa': data['name'],
    }
    return result


def _handle_bsbuilder(args):
    # TODO implement
    return data['bsbuilder']


def _handle_image_builder(args):
    # TODO implement
    return data['image_builder']


def _handle_publisher(args):
    # TODO implement
    return data['publisher']


def _handle_test_runner(args):
    # TODO implement
    return data['test_runner']


def _get_config(args):
    with open(args.input_file) as f:
        if args.input_format == 'json':
            config = json.loads(f.read())
        else:
            config = yaml.safe_load(f)
    return config


def _get_service_handlers():
    '''get the availble service handlers.

    This is a handy mechanism so we don't have to maintain a list of available
    service handlers. Any function prefixed with '_handle_' will be exposed to
    the user.
    '''
    return [x[8:] for x in globals() if x.startswith('_handle_')]


def _get_parser():
    parser = argparse.ArgumentParser(
        description='Wraps the REST API calls with a progress queue monitor.')
    parser.add_argument('--service',
                        required=True,
                        choices=_get_service_handlers(),
                        help='The service name to call.')
    parser.add_argument('--input-file',
                        required=True,
                        help='File containing all of the input parameters.')
    parser.add_argument('--input-format',
                        required=False,
                        choices=format_list,
                        default='json')
    parser.add_argument('--output-file',
                        required=True,
                        help='File to store the API call results.')
    parser.add_argument('--output-format',
                        required=False,
                        choices=format_list,
                        default='json')
    return parser


def main(args):
    handler = globals()['_handle_' + args.service]
    logger.debug('Executing service wrapper: {}'.format(args.service))

    try:
        results = handler(args)
    except urllib2.HTTPError as e:
        body = e.read()
        logging.exception('Unable to handle request: %s: %s', e, body)
        results = {
            'result': 'ERROR',
            'resp_code': e.reason,
            'resp_body': body,
        }
    except Exception as e:
        logging.exception('Unexpected error: %s', e)
        results = {
            'result': 'FAILED',
            'error': e.message,
        }

    with open(args.output_file, 'w') as output_file:
        out_data = None
        if args.output_format == 'json':
            out_data = json.dumps(results)
        elif args.output_format == 'yaml':
            out_data = yaml.dump(results)
        output_file.write('{}\n'.format(out_data))


if __name__ == '__main__':
    args = _get_parser().parse_args()
    main(args)
