# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import os
import StringIO
import sys
import tempfile
import unittest
import urllib2

import mock

sys.path.append(os.path.join(os.path.dirname(__file__), '../../bin'))
import lander_service_wrapper


class TestWrapper(unittest.TestCase):
    '''Sanity tests of the lander_service_wrapper.py script.'''

    def setUp(self):
        super(TestWrapper, self).setUp()

        self.results = tempfile.mkstemp(prefix='lander-test-')[1]
        self.addCleanup(os.unlink, self.results)

        self.params = tempfile.mkstemp(prefix='lander-test-')[1]
        self.addCleanup(os.unlink, self.params)
        data = {
            'request_id': 1,
            'ppa_assigner_url': 'foo',
        }
        with open(self.params, 'w') as f:
            f.write(json.dumps(data))

        self.base_args = [
            '--input-file', self.params, '--output-file', self.results]

    def testBadService(self):
        '''make sure we exit gracefully for a bad service name'''
        args = self.base_args + ['--service', 'bad-name']
        with self.assertRaises(SystemExit):
            lander_service_wrapper._get_parser().parse_args(args)

    @mock.patch('lander_service_wrapper._post')
    def testPPAAssigner404(self, post):
        '''Ensure a "no ppas" type error generates a response'''
        err_msg = 'foo bar'
        post.side_effect = urllib2.HTTPError(
            'url', 404, '404', None, StringIO.StringIO(err_msg))

        args = self.base_args + ['--service', 'ppa_assigner']
        args = lander_service_wrapper._get_parser().parse_args(args)
        lander_service_wrapper.main(args)

        with open(self.results) as f:
            data = json.loads(f.read())
            self.assertEqual('ERROR', data['result'])
            self.assertEqual(err_msg, data['resp_body'])
            self.assertEqual('404', data['resp_code'])

    @mock.patch('lander_service_wrapper._post')
    def testPPAAssignerUnexpected(self, post):
        '''Ensure unexpected errors still generate a response'''
        err_msg = 'foo bar'
        post.side_effect = RuntimeError(err_msg)

        args = self.base_args + ['--service', 'ppa_assigner']
        args = lander_service_wrapper._get_parser().parse_args(args)
        lander_service_wrapper.main(args)

        with open(self.results) as f:
            data = json.loads(f.read())
            self.assertEqual('FAILED', data['result'])
            self.assertEqual(err_msg, data['error'])

    @mock.patch('lander_service_wrapper._post')
    @mock.patch('lander_service_wrapper._get')
    def testPPAAssignerSucceeds(self, get, post):
        '''Ensure we generate the proper response if things succeed.'''
        ppa = 'ppa:foo'
        post.return_value = 'http://foo/api/v1/ppa/1'
        get.return_value = {'name': ppa}

        args = self.base_args + ['--service', 'ppa_assigner']
        args = lander_service_wrapper._get_parser().parse_args(args)
        lander_service_wrapper.main(args)

        with open(self.results) as f:
            data = json.loads(f.read())
            self.assertEqual('PASSED', data['result'])
            self.assertEqual(ppa, data['ppa'])
